/* seqpp/PMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief   Parcimonious Markov model

   01/2004
*/

#ifndef SEQPP_PMARKOV_H
#define SEQPP_PMARKOV_H

#include <seqpp/Markov.h>
#include <seqpp/PhasedPMarkov.h>
#include <seqpp/pmm_tree.h>

/**
 * \ingroup General_Use_Markov
 */

/*!  \class PMarkov seqpp/PMarkov.h
  \brief  Parcimonious Markov modelling, estimation and simulation.
  
  This is a special case of a phased Parcimonious Markov [PhasedPMarkov] model when only one phase is considered, and also a Markov object.
*/

class PMarkov :  public Markov
{
 protected :
  //! Parcimonious Context Tree
  pmm_tree * _pmmtree;

 public:
  //! Constructor 1 from a SequenceSet
  /*!
    \param  seqset a set of sequences for estimation
    \param prior_alpha alpha for the a priori law, by default -1 means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PMarkov( const SequenceSet & seqset,
	   double prior_alpha = -1, 
       	 double penalty = 0.,
	   const string & xmlfile = string() )
    :  Markov( seqset.tell_alphabet_size(), seqset.tell_order() )
  { 
    Partition p( seqset.tell_alphabet_size() );
    seqset.count_occurencies(); 
    estimate( seqset.get_count(), true, seqset.get_translator(), p, prior_alpha, penalty, xmlfile );
  }

  //! Constructor 2 from a Sequence
  /*!
    \param  seq sequence for estimation
    \param prior_alpha alpha for the a priori law, by default -1 means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PMarkov( const Sequence & seq, 
	   double prior_alpha = -1,
	   double penalty = 0.,
	   const Translator & trans = Translator(),
	   const string & xmlfile = string() )
    : Markov(  seq.tell_alphabet_size(), seq.tell_order() )
  { 
    Partition p( seq.tell_alphabet_size() );
    seq.count_occurencies(); 
    estimate( seq.get_count(), true, trans, p, prior_alpha, penalty, xmlfile );
  }

   //! Constructor 3 from a SequenceSet and a Partition -file
  /*!
    \param partitionfile file containing a selected partition, when default overall partition is too heavy
    \param seqset a set of sequences for estimation
    \param prior_alpha alpha for the a priori law, by default -1 means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
    <br>Exple of partitionfile (grouped tokens, separed with a "|", beware the spaces) with 4 partitions<br>
    \code
     a c | g t
     a c | g | t
     a | c | g | t
     c g t | a
    \endcode
  */
  PMarkov( const string & partitionfile,
	   const SequenceSet & seqset, 
	   double prior_alpha = -1,
	   double penalty = 0.,
	   const string & xmlfile = string() )
    : Markov(  seqset.tell_alphabet_size(), seqset.tell_order() )
  {
    Partition p( seqset.get_translator(), partitionfile );
    seqset.count_occurencies(); 
    estimate( seqset.get_count(), true, seqset.get_translator(), p, prior_alpha, penalty, xmlfile ); 
  }

  //! Constructor 4 from a Sequence and a Partition -file
  /*! 
    \param partitionfile file containing a selected partition, when default overall partition is too heavy
    \param  seq sequence for estimation
    \param trans a Translator is required for the partition reading
    \param prior_alpha alpha for the a priori law, by default -1 means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
    <br>Exple of partitionfile (grouped tokens, separed with a "|", beware the spaces) with 4 partitions<br>
    \code
    a c | g t
    a c | g | t
    a | c | g | t
    c g t | a
    \endcode
  */
  PMarkov( const string & partitionfile,
	   const Translator & trans, 
	   const Sequence & seq,
	   double prior_alpha = -1,
	   double penalty = 0.,
	   const string & xmlfile = string() )
    : Markov(  seq.tell_alphabet_size(), seq.tell_order() )
  {
    Partition p( trans, partitionfile );
    seq.count_occurencies(); 
    estimate( seq.get_count(), true, trans, p, prior_alpha, penalty, xmlfile );    
 }

   //! Constructor 5 from a coded-word count
  /*!
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param size alphabet size
    \param order markovian order associated to the word count
    \param prior_alpha alpha for the a priori law, by default -1 means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PMarkov( unsigned long * count,
	   short size, short order,
	   double prior_alpha = -1,
	   double penalty = 0., 	   
	   const Translator & trans = Translator(),	   
	   const string & xmlfile = string() )
    : Markov( size, order )    
  {
    //cout<<"PMarkov constr."<<endl;
    Partition p( size );   
    estimate( count, false, trans, p, prior_alpha, penalty, xmlfile );
  }

 //! Constructor 6 from  a coded-word count
  /*! 
    \param partitionfile file containing a selected partition, when default overall partition is too heavy
    \param trans a Translator is required for the partition reading
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param order markovian order associated to the word count
    \param prior_alpha alpha for the a priori law, by default -1 means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
    <br>Exple of partitionfile (grouped tokens, separed with a "|", beware the spaces) with 4 partitions<br>
    \code
    a c | g t
    a c | g | t
    a | c | g | t
    c g t | a
    \endcode
  */
  PMarkov( const string & partitionfile,
	   const Translator & trans,
	   unsigned long * count, 
	   short order,
	   double prior_alpha = -1,
	   double penalty = 0., 
	   const string & xmlfile = string() )
    : Markov( trans.tell_alphabet_size(), order )
  {
    Partition p( trans, partitionfile );
    estimate( count, false, trans, p, prior_alpha, penalty, xmlfile );    
  }


  //! performs the estimation [used in each constructor] 
  void estimate( unsigned long * count, bool decal_required, 
		 const Translator & trans, Partition & p, 
		 double prior_alpha ,
		 double penalty = 0.,
		 const string & xmlfile = string() )
  {
   _pmmtree = new pmm_tree( _size, p, _order, prior_alpha );
   _nb_param = 0;   
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node);
     _pmmtree->select( count,
		       decal_required, p );
     _pmmtree->tree_to_matrix( _Pi );
     //cout<<"nbleaves: "<<_pmmtree->nb_leaves()<<endl;
     _nb_param += _pmmtree->nb_leaves() * (_size-1);
     std::stringstream treenamestream;
     _pmmtree->save( trans, root_node, treenamestream.str() );
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{
     _pmmtree->select( count,
		       decal_required, p );
     
     _pmmtree->tree_to_matrix( _Pi );
     _nb_param += _pmmtree->nb_leaves() * (_size-1);
   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   } 
   _pmmtree->select( count,
		     decal_required, p );   
   _pmmtree->tree_to_matrix( _Pi );
   _nb_param += _pmmtree->nb_leaves() * (_size-1);
#endif
      
   delete _pmmtree;
  }


  ~PMarkov(){
    //cout<<"destr.Pmarkov"<<endl;
  }
};
#endif
