/* seqpp/PhasedPMarkov.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*!
 \file
  \author Vincent Miele
  \brief  Phased Parcimonious Markov model

   01/2004
*/

#ifndef SEQPP_PHASEDPMARKOV_H
#define SEQPP_PHASEDPMARKOV_H

#include <seqpp/PhasedMarkov.h>
#include <seqpp/pmm_tree.h>

/**
 * \ingroup General_Use_Markov
 */

/*!  \class PhasedPMarkov seqpp/PhasedPMarkov.h
  \brief   Phased Parcimonious Markov modelling, estimation and simulation.

  PhasedPMarkov is a PhasedMarkov object with a different estimation step.
  This object performs the estimation with the Parcimonious Markov algorithm 
  and then transfoms, once per phase, the parcimonious context tree in a markovian matrix. 
  xml outputs can be activated to save the associated trees.
*/
class PhasedPMarkov : public PhasedMarkov
{
 protected : 
  //! Parcimonious Context Trees
  pmm_tree * _pmmtree;

 public:
  //! Constructor 1 from a SequenceSet
  /*!
    \param  seqset a set of sequences for estimation
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha alpha for the a priori law, by default -1. means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PhasedPMarkov( const SequenceSet & seqset,
		 short phase, short initial_phase = 0,
		 double prior_alpha = -1.,
		 double penalty = 0.,
		 const string & xmlfile = string() )
    :  PhasedMarkov(  seqset.tell_alphabet_size(), seqset.tell_order(), phase )
  { 
    Partition p( seqset.tell_alphabet_size() );
    seqset.count_p_occurencies( phase, initial_phase ); 
    estimate( seqset.get_p_count(), true, seqset.get_translator(), p, phase, initial_phase, prior_alpha, penalty, xmlfile );
  }
  //! Constructor 2 from a Sequence
  /*!
    \param  seq sequence for estimation
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha alpha for the a priori law, by default -1. means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PhasedPMarkov( const Sequence & seq,
		 short phase, short initial_phase = 0,
		 double prior_alpha = -1.,
		 double penalty = 0., 
		 const Translator & trans = Translator(),
		 const string & xmlfile = string() )
    : PhasedMarkov(  seq.tell_alphabet_size(), seq.tell_order(), phase )
  {
    Partition p( seq.tell_alphabet_size() );
    seq.count_p_occurencies( phase, initial_phase ); 
    estimate( seq.get_p_count(), true, trans, p, phase, initial_phase, prior_alpha, penalty, xmlfile );
  }
  
  //! Constructor 3 from a SequenceSet and a Partition -file
  /*!
    \param partitionfile file containing a selected partition, when default overall partition is too heavy
    \param seqset a set of sequences for estimation
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha alpha for the a priori law, by default -1. means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
    <br>Exple of partitionfile (grouped tokens, separed with a "|", beware the spaces) with 4 partitions<br>
    \code
     a c | g t
     a c | g | t
     a | c | g | t
     c g t | a
    \endcode
  */
  PhasedPMarkov( const string & partitionfile,
		 const SequenceSet & seqset,
		 short phase, short initial_phase = 0,
		 double prior_alpha = -1.,
		 double penalty = 0.,
		 const string & xmlfile = string() )
    :  PhasedMarkov(  seqset.tell_alphabet_size(), seqset.tell_order(), phase )
  {     
    Partition p( seqset.get_translator(), partitionfile ); 
    seqset.count_p_occurencies( phase, initial_phase ); 
    estimate( seqset.get_p_count(), true, seqset.get_translator(), p, phase, initial_phase, prior_alpha, penalty ,xmlfile );
  }

  //! Constructor 4 from a Sequence and a Partition -file
  /*! 
    \param partitionfile file containing a selected partition, when default overall partition is too heavy
    \param trans a Translator is required for the partition reading
    \param  seq sequence for estimation
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha alpha for the a priori law, by default -1. means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
    <br>Exple of partitionfile (grouped tokens, separed with a "|", beware the spaces) with 4 partitions<br>
    \code
    a c | g t
    a c | g | t
    a | c | g | t
    c g t | a
    \endcode
  */
  PhasedPMarkov( const string & partitionfile,
		 const Translator & trans,
		 const Sequence & seq,
		 short phase, short initial_phase = 0,
		 double prior_alpha = -1.,
		 double penalty = 0., 
		 const string & xmlfile = string() )
     :  PhasedMarkov(  seq.tell_alphabet_size(), seq.tell_order(), phase )
  {
    Partition p( trans, partitionfile );
    seq.count_p_occurencies( phase, initial_phase ); 
    estimate( seq.get_p_count(), true, trans, p, phase, initial_phase,  prior_alpha, penalty, xmlfile );    
  }

  //! Constructor 5 from a coded-word count
  /*!
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param size alphabet size
    \param order markovian order associated to the word count
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha alpha for the a priori law, by default -1. means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param trans a Translator is required only for the xml saving
    \param xmlfile xmlfile for tree saving (if xml2 activated)
  */
  PhasedPMarkov( unsigned long * * count,
		 short size, short order,
		 short phase, short initial_phase = 0,
		 double prior_alpha = -1., 
		 double penalty = 0.,
		 const Translator & trans = Translator(),
		 const string & xmlfile = string() )
    :  PhasedMarkov( size, order, phase )
  {
    //cout<<"PhasedPMarkov constr. 5"<<endl;
    Partition p( size );   
    estimate( count, false, trans, p, phase, initial_phase, prior_alpha, penalty, xmlfile );
  }

  //! Constructor 6 from a coded-word count and a Partition -file
  /*! 
    \param partitionfile file containing a selected partition, when default overall partition is too heavy
    \param trans a Translator is required for the partition reading
    \param count count of all the coded word(base size) of size order+1 for each phase, for estimation
    \param order markovian order associated to the word count
    \param phase phase 
    \param initial_phase phase of the first element of each sequence
    \param prior_alpha alpha for the a priori law, by default -1. means 1./alphabet_size
    \param penalty penalty on the leaves number, by default 0  
    \param xmlfile xmlfile for tree saving (if xml2 activated)
    <br>Exple of partitionfile (grouped tokens, separed with a "|", beware the spaces) with 4 partitions<br>
    \code
    a c | g t
    a c | g | t
    a | c | g | t
    c g t | a
    \endcode
  */
  PhasedPMarkov( const string & partitionfile,
		 const Translator & trans,
		 unsigned long * * count,
		 short order,
		 short phase, short initial_phase = 0,
		 double prior_alpha = -1., 
		 double penalty = 0.,
		 const string & xmlfile = string() )
     :  PhasedMarkov(  trans.tell_alphabet_size(), order, phase )
  {
    Partition p( trans, partitionfile );
    estimate( count, false, trans, p, phase, initial_phase, prior_alpha, penalty, xmlfile );    
  }


  //! performs the estimation [used by each constructor] 
  void estimate( unsigned long * * count, bool decal_required, 
		 const Translator & trans, Partition & p,
		 short phase, short initial_phase, 
		 double prior_alpha ,
		 double penalty ,
		 const string & xmlfile )
 {
   _pmmtree = new pmm_tree( _size, p, _order, prior_alpha, penalty );
   
   /*
   _pmmtree->expanded_tree(*count, decal_required, p);
   exit(1);
   */

   /*
     cout<<"TIME cost in 0( ("<< p.tell_nbposs()<<"^"<<_order 
     <<"*("<<p.tell_cardinal()<<"+1) )"
     <<endl; 
   */
   //cout<<"nb poss: "<<p.tell_nbposs()<<" cardinal: "<<p.tell_cardinal()<<endl;

   _nb_param = 0;   
#ifdef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     xmlDocPtr doc=xmlNewDoc(BAD_CAST "1.0");
     xmlNodePtr root_node=xmlNewNode(NULL,BAD_CAST "treeset");
     xmlDocSetRootElement(doc,root_node); 
     for (int i=0; i<phase; i++){ 
       cout<<"In select"<<endl;
       _pmmtree->select( count[i],
			 decal_required, p ); 
       cout<<"In tree2matrix"<<endl;
       _pmmtree->tree_to_matrix( _Pis[i] );
       //cout<<"nbleaves: "<<_pmmtree->nb_leaves()<<endl;
       _nb_param += _pmmtree->nb_leaves() * (_size-1);
       std::stringstream treenamestream;
       treenamestream <<"phase "<<i;
       _pmmtree->save( trans, root_node, treenamestream.str() );
     }
     xmlSaveFormatFileEnc(xmlfile.c_str(),doc,"ISO-8859-1",1);
     xmlFreeDoc(doc); 
   }
   else{
     for (int i=0; i<phase; i++){
       _pmmtree->select( count[i],
			 decal_required, p );
       
       _pmmtree->tree_to_matrix( _Pis[i] );
       
       _nb_param += _pmmtree->nb_leaves() * (_size-1);
     }
   }
#endif
#ifndef HAVE_LIBXML2
   if (xmlfile.size() != 0){
     cerr<<"in PhasedPMarkov: no libxml2 detected to fill "<<xmlfile<<endl;
   }
   for (int i=0; i<phase; i++){    
     _pmmtree->select( count[i],
		       decal_required, p );
     _pmmtree->tree_to_matrix( _Pis[i] );

     _nb_param += _pmmtree->nb_leaves() * (_size-1);
   }
#endif
   
   delete _pmmtree;
 }
 
//! Destructor
 ~PhasedPMarkov(){} 
};
#endif
