/* seqpp/pmm_tree.h
 *
 * Copyright (C) 2003 Laboratoire Statistique & Gnome
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*!
  \internal
  \file pmm_tree.h
  \author P.-Y. Bourguignon-V.Miele-M.Hoebeke
  \brief Header for the parsimonious Markov tree class
*/


#ifndef SEQPP_PMM_TREE_H
#define SEQPP_PMM_TREE_H

#include <seqpp/pmm_node.h>
#include <seqpp/pmm_leaf.h>
#include <seqpp/SequenceSet.h>
#include <seqpp/Partition.h>
#include <string>
#include <gsl/gsl_sf.h>

#ifdef HAVE_LIBXML2
#include <libxml/tree.h>
#endif
using namespace std ;


/*!
  \internal
  \class pmm_tree seqpp/pmm_tree.h
  \brief estimation tree for Parcimonious Markov model selection 
*/

class pmm_tree {
 public:
  /*!
    \brief Constructor
    Only builds the root
    \param alphabet_size size of the alphabet
    \param synonymous vector of synonymous tokens for the predictors
    \param depth The depth of the tree to build
    \param prior_alpha alpha for the a priori law, by default -1 means 1./alphabet_size.
  */
  pmm_tree::pmm_tree( short alphabet_size, 
		      Partition & partition,
		      int depth, 
		      double prior_alpha = -1.,
		      double penalty = 0. );

  //! Destructor
  ~pmm_tree() ;

  /*!
    \brief Performs maximal a posteriori selection
    \param count is the word count
    \param decal_required necessary when using a count of word from 1-word to (_max_depth+1)-word
  */
  double select( const unsigned long * count, bool decal_required, 
		 Partition & partition );
  
  /*!-------------------------------------------------------------
   * \brief Returns the current number of leaves
   */
  int nb_leaves() const{
    return _nbleaves;
  }

  /*!
   * \brief Deduces the markov matrix (stored in line) from the pmm
   */
  void tree_to_matrix( double* mat );
   

#ifdef HAVE_LIBXML2  
  void save( const Translator & trans, 
	     xmlNodePtr parent_node, const string &treename );
#endif

 private:
  
  short _alphabet_size;
  //!< A pointer to the root node
  pmm_node * _root ;
  //!< The depth of the tree
  int _depth;
  //!< Number of leaves
  int _nbleaves;        
  //! alpha for the prior law
  double _prior_alpha;
  //! penalty on the leaves number
  double _penalty;

  // Optimization Tools ----------------------------
  // code("A", i) = code("A") en position i sur depth+1 posit
  int ** _code;
  //! count des depth+1-mots par ordre alphabetique
  vector<unsigned long> _count;

  int * _codeinterm;
  Partition::const_iterator _it;
  Partition::const_iterator _end;
  Partition::const_iterator _bestpart;

  vector<short>::iterator * _tab;
  vector<short>::iterator * _tabend;

  vector< vector<short> >  _synonymous;
  short _syn_size;
  short * _corresp;

  vector< vector<short> > _list;//! list of possible elt in a partition
  int _motifdefault;//! index in list of the default whole alphabet element
  short _motifdefault_skip;//! check if default whole alphabet element must be skipped
  
  //---------------------------------------------------------
  /*!
    \brief Recursive step of the select() method
  */
  double select_step(node_base * noeud, Partition & partition) ;
  
  double complete(node_base * noeud) ;
 
  /*!
    \brief Counts the motif given by path in the specified count, with the next alphabet_size chars as last letter
     \param path the motif to count 
     \param counts vector of count for each next letter
  */
  long count_hmotif( vector< int > & path, 
		     vector<long> & counts, short howmany_next ) ;
  
  inline pmm_node & getRoot() { 
    return *_root ; 
  };
  pmm_leaf & getLeaf(vector<short> & rpath) ;

  pmm_leaf & getLeaf(unsigned long code) ;

  //! prune the tree under n
  void prune( pmm_node * n );


  //---------------------------------------------------------
  void set_code();

  void set_count( const unsigned long * count, bool decal_required );
  void set_count_r( const unsigned long * count, 
		    unsigned long code, unsigned long codesyn, short depth );
  //---------------------------------------------------------
 //! compute the number of word associated to a path
  int nbword_in_vmotif( const vector< int > & path ){
    short i;
    int nb = 1;
    for ( i=0; i<_depth; i++)
      nb *= _list[ path[i] ].size();
    return nb;
  }
};
#endif
